﻿using System;
using System.Reflection;
using HarmonyLib;
using Verse;
using RimWorld;
using RimWorld.Planet;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;

namespace YayoPollution
{
    
    
    

    public class HarmonyPatches : Mod
    {

        public HarmonyPatches(ModContentPack content) : base(content)
        {
            var harmony = new Harmony("com.yayo.pollution");
            harmony.PatchAll(Assembly.GetExecutingAssembly());

        }

    }



    [HarmonyPatch(typeof(WorldObject))]
    [HarmonyPatch("ExposeData")]
    public class patch_WorldObject_exposeData
    {
        private static void Postfix(WorldObject __instance)
        {
            dataUtility.GetData(__instance).ExposeData();
        }
    }

    [HarmonyPatch(typeof(World))]
    [HarmonyPatch("ExposeData")]
    public class patch_World_exposeData
    {
        private static void Postfix(World __instance)
        {
            dataUtility.GetData(__instance).ExposeData();
        }
    }









    [HarmonyPatch(typeof(WorldPollutionUtility), "PolluteWorldAtTile")]
    public class Patch_WorldPollutionUtility_PolluteWorldAtTile
    {
        public static bool Prefix(int root, float pollutionAmount, HashSet<Faction> ___tmpSeenFactions)
        {
            
            //Log.Message("Patch_WorldPollutionUtility_PolluteWorldAtTile");
            if (root == -1)
            {
                core.pollutionAlram = true;
                return false;
            }
            ___tmpSeenFactions.Clear();
            int bestTile = WorldPollutionUtility.FindBestTileToPollute(root);
            if (bestTile != -1)
            {
                Tile tile = Find.WorldGrid[bestTile];
                float pollution = tile.pollution;
                _ = tile.pollution;
                float targetPollution = tile.pollution + pollutionAmount;
                float leftPollution = targetPollution - 1f;
                tile.pollution = Mathf.Clamp01(targetPollution);
                MapParent mapParent = Find.WorldObjects.MapParentAt(bestTile);
                if (core.pollutionAlram && (mapParent == null || !mapParent.HasMap))
                {
                    Vector2 vector = Find.WorldGrid.LongLatOf(bestTile);
                    string value = vector.y.ToStringLatitude() + " / " + vector.x.ToStringLongitude();
                    Messages.Message("MessageWorldTilePollutionChanged".Translate(pollutionAmount.ToStringPercent(), value), new LookTargets(bestTile), MessageTypeDefOf.NegativeEvent, false);
                }
                Map map = Current.Game.FindMap(bestTile);

                // yayo
                if (pollution < 0.999f && map != null)
                {
                    core.PolluteMapToPercent(map, tile.pollution);
                }
                //

                Find.World.renderer.Notify_TilePollutionChanged(bestTile);
                ___tmpSeenFactions.Clear();
                //if (leftPollution > 0f)
                //{
                //    PolluteWorldAtTile(bestTile, leftPollution);
                //}
            }

            core.pollutionAlram = true;
            return false;
            
        }

    }


    

    //[HarmonyPatch(typeof(WorldPollutionUtility), "FindBestTileToPollute")]
    //public class Patch_WorldPollutionUtility_FindBestTileToPollute
    //{
    //    public static bool Prefix(ref int __result, int root, HashSet<Faction> ___tmpSeenFactions, List<int> ___tmpPossiblePollutableTiles, List<int> ___tmpTileNeighbors)
    //    {
    //        //Log.Message("Patch_WorldPollutionUtility_FindBestTileToPollute");
    //        if (root == -1)
    //        {
    //            reset();
    //            __result = -1;
    //            return false;
    //        }
    //        World world = Find.World;
    //        WorldGrid grid = world.grid;
    //        _ = grid[root];
    //        if (CanPollute(root))
    //        {
    //            reset();
    //            __result = root;
    //            return false;
    //        }
    //        ___tmpPossiblePollutableTiles.Clear();
    //        int bestDistance = int.MaxValue;
    //        Find.WorldFloodFiller.FloodFill(root, (int x) => !CanPollute(x), delegate (int t, int d)
    //        {
    //            ___tmpTileNeighbors.Clear();
    //            grid.GetTileNeighbors(t, ___tmpTileNeighbors);
    //            for (int i = 0; i < ___tmpTileNeighbors.Count; i++)
    //            {
    //                if (CanPollute(___tmpTileNeighbors[i]) && !___tmpPossiblePollutableTiles.Contains(___tmpTileNeighbors[i]))
    //                {
    //                    int num = Mathf.RoundToInt(grid.ApproxDistanceInTiles(root, ___tmpTileNeighbors[i]));
    //                    if (num <= bestDistance)
    //                    {
    //                        bestDistance = num;
    //                        ___tmpPossiblePollutableTiles.Add(___tmpTileNeighbors[i]);
    //                        ___tmpPossiblePollutableTiles.RemoveAll((int u) => Mathf.RoundToInt(grid.ApproxDistanceInTiles(root, u)) > bestDistance);
    //                    }
    //                }
    //            }
    //            reset();
    //            return false;
    //        });
    //        int found = (from t in ___tmpPossiblePollutableTiles
    //                     orderby grid[t].PollutionLevel(), grid[t].pollution descending
    //                     select t).FirstOrFallback(-1);
    //        ___tmpPossiblePollutableTiles.RemoveAll((int t) => grid[t].PollutionLevel() > grid[found].PollutionLevel() && grid[t].pollution < grid[found].pollution);
    //        found = ___tmpPossiblePollutableTiles.RandomElement();
    //        ___tmpPossiblePollutableTiles.Clear();
    //        ___tmpTileNeighbors.Clear();

    //        //Log.Message($"cell: {found}, dist: {grid.ApproxDistanceInTiles(root, found)}");
    //        //Log.Message($"cell: {found}, dist: {Vector3.Distance(grid.GetTileCenter(root), grid.GetTileCenter(found))}");
    //        // yayo
    //        if (found == 0)
    //        {
    //            reset();
    //            __result = root;
    //            return false;
    //        }


    //        //
    //        reset();
    //        __result = found;
    //        return false;

    //        // yayo
    //        bool CanPollute(int t)
    //        {
    //            if(core.pollutionIgnoreRoot && t == root)
    //            {
    //                return false;
    //            }
    //            else if (grid[t].biome.allowPollution)
    //            {
    //                // yayo
    //                return grid[t].pollution < core.targetPollution;

    //            }
    //            return false;
    //        }
    //        void reset()
    //        {
    //            //core.targetPollution = 0.5f;
    //            //core.pollutionIgnoreRoot = false;
    //        }
    //        //
            

    //    }

    //}








    //[HarmonyPatch(typeof(Map))]
    //[HarmonyPatch("ExposeData")]
    //public class patch_map_exposeData
    //{
    //    private static void Postfix(Map __instance)
    //    {
    //        dataUtility.GetData(__instance).ExposeData();
    //    }
    //}

    //[HarmonyPatch(typeof(World))]
    //[HarmonyPatch("ExposeData")]
    //public class patch_World_exposeData
    //{
    //    private static void Postfix(World __instance)
    //    {
    //        dataUtility.GetData(__instance).ExposeData();
    //    }
    //}





    //// 월드 오브젝트 생성시

    //[HarmonyPatch(typeof(WorldObjectsHolder))]
    //[HarmonyPatch("Add")]
    //public class patch_WorldObjectsHolder_Add
    //{
    //    private static void Postfix(WorldObjectsHolder __instance, WorldObject o)
    //    {
    //        if(core.val_worldBiome && Current.ProgramState == ProgramState.Playing)
    //        {
    //            if(o is Site || o is Settlement)
    //            {
    //                BiomeDef b = core.getRandomBiome();
    //                Find.WorldGrid[o.Tile].biome = b;
    //                Find.WorldGrid[o.Tile].temperature = core.getBiomeTemp(b);
    //            }
    //        }

    //    }
    //}















}
