﻿using UnityEngine;
using Verse;
using RimWorld;
using RimWorld.Planet;
using HugsLib;
using HugsLib.Settings;
using System.Linq;
using RimWorld.Planet;
using System.Collections.Generic;
using HarmonyLib;
using System;
using UnityEngine.SceneManagement;

namespace YayoPollution
{

    public class core : ModBase
    {
        public override string ModIdentifier => "YayoPollution";

        public static bool using_advBiome = false;

        static core()
        {
            
        }



        // 5년 테스트
        private SettingHandle<float> val_simSpeed_s;
        public static float val_simSpeed;

        // 오염지점 수
        private SettingHandle<int> val_targetCount_s;
        public static int val_targetCount;

        // 오염지점 선정 주기 (일)
        private SettingHandle<float> val_targetSetupDelay_s;
        public static float val_targetSetupDelay; 
        
        // 플레이어 주변에 오염지점 생성 확률
        private SettingHandle<float> val_nearPollutionPer_s;
        public static float val_nearPollutionPer;

        // 하루에 오염지점의 오염도가 증가하는 양 (%)
        private SettingHandle<float> val_targetPollution_s;
        public static float val_targetPollution;
        
        // 월드맵에서 하루에 오염이 확산되는 주변 타일의 수 (타일/일)
        private SettingHandle<float> val_waveSpeed_s;
        public static float val_waveSpeed;
        
        // 월드맵에서의 오염 확산 웨이브 생성 단위 (%)
        private SettingHandle<float> val_waveGenUnit_s;
        public static float val_waveGenUnit = 5f;

        // 로그
        private SettingHandle<bool> val_useLog_s;
        public static bool val_useLog;

        



        static public List<FactionDef> ar_faction = new List<FactionDef>();
        static public List<SettingHandle<bool>> ar_faction_useSetting = new List<SettingHandle<bool>>();
        static public List<bool> ar_faction_use = new List<bool>();
        

        public override void DefsLoaded()
        {
            val_targetCount_s = Settings.GetHandle<int>("val_targetCount", "val_targetCount_t".Translate(), "val_targetCount_d".Translate(), 6);
            val_targetSetupDelay_s = Settings.GetHandle<float>("val_targetSetupDelay", "val_targetSetupDelay_t".Translate(), "val_targetSetupDelay_d".Translate(), 20f);
            val_nearPollutionPer_s = Settings.GetHandle<float>("val_nearPollutionPer", "val_nearPollutionPer_t".Translate(), "val_nearPollutionPer_d".Translate(), 20f);
            val_targetPollution_s = Settings.GetHandle<float>("val_targetPollution", "val_targetPollution_t".Translate(), "val_targetPollution_d".Translate(), 0.25f);
            val_waveSpeed_s = Settings.GetHandle<float>("val_waveSpeed", "val_waveSpeed_t".Translate(), "val_waveSpeed_d".Translate(), 66.66f);
            val_waveGenUnit_s = Settings.GetHandle<float>("val_waveGenUnit", "val_waveGenUnit_t".Translate(), "val_waveGenUnit_d".Translate(), 5f);
            
            //Setup and get settings values
            foreach (FactionDef faction in from faction in DefDatabase<FactionDef>.AllDefs where
                !faction.hidden &&
                !faction.isPlayer
            select faction
            )
            {
                ar_faction.Add(faction);
            }
            
            for (int i = 0; i < ar_faction.Count; i++)
            {
                FactionDef f = ar_faction[i];
                bool defaultVal = f == FactionDefOf.PirateWaster;
                ar_faction_useSetting.Add(Settings.GetHandle<bool>($"{f.defName}", ($"{f.label} ({f.defName})").Colorize(f.DefaultColor), "", defaultVal));
                ar_faction_use.Add(ar_faction_useSetting[i].Value);
                
            }
            
            val_useLog_s = Settings.GetHandle<bool>("val_useLog", "val_useLog_t".Translate(), "val_useLog_d".Translate(), false);
            val_simSpeed_s = Settings.GetHandle<float>("val_simSpeed", "val_simSpeed_t".Translate(), "val_simSpeed_d".Translate(), 1f);
            

            SettingsChanged();
        }

        public override void SettingsChanged()
        {
            val_targetCount = Mathf.Max(val_targetCount_s.Value, -1);
            val_targetCount_s.Value = val_targetCount;
            val_targetSetupDelay = Mathf.Max(val_targetSetupDelay_s.Value, 0.0001f);
            val_targetSetupDelay_s.Value = val_targetSetupDelay;
            val_nearPollutionPer = Mathf.Clamp(val_nearPollutionPer_s.Value, 0f, 100f);
            val_nearPollutionPer_s.Value = val_nearPollutionPer;
            val_targetPollution = Mathf.Max(val_targetPollution_s.Value, 0.0001f);
            val_targetPollution_s.Value = val_targetPollution;
            val_waveSpeed = Mathf.Clamp(val_waveSpeed_s.Value, 0.0001f, GenDate.TicksPerDay);
            val_waveSpeed_s.Value = val_waveSpeed;
            val_waveGenUnit = Mathf.Clamp(val_waveGenUnit_s.Value, 5f, 100f);
            val_waveGenUnit_s.Value = val_waveGenUnit;
            
            for(int i = 0; i < ar_faction_use.Count; i++)
            {
                ar_faction_use[i] = ar_faction_useSetting[i].Value;
            }

            val_useLog = val_useLog_s.Value;
            val_simSpeed = val_simSpeed_s.Value;
            
            val_waveGenUnitPer = val_waveGenUnit * 0.01f;
            tickFindNewPollutionTarget = Mathf.Max(Mathf.RoundToInt(val_targetSetupDelay / val_simSpeed * GenDate.TicksPerDay), 1);
            tickGap = Mathf.Max(Mathf.RoundToInt(GenDate.TicksPerDay / (val_waveSpeed * val_simSpeed)), 1);
            targetPollutionAmt = Mathf.Clamp(((val_targetPollution * val_simSpeed) / 100f) / (GenDate.TicksPerDay / tickGap), 0f, 1f);

            if (val_useLog)
            {
                Log.Message($"# tickFindNewPollutionTarget: {tickFindNewPollutionTarget}");
                Log.Message($"# tickGap: {tickGap}");
                Log.Message($"# targetPollutionAmt: {targetPollutionAmt}");

            }
        }
        
        

        public override void SceneLoaded(Scene scene)
        {
            base.SceneLoaded(scene);
            dataUtility.dic_yyPollution.Clear();
            dataUtility.dic_clearup.Clear();
        }

        public override void WorldLoaded()
        {
            base.WorldLoaded();
            if (val_useLog) Log.Message($"# World.ConstantRandSeed : {Find.World.ConstantRandSeed}");
        }



        // -----------------------------------------


        static public int tickGame = 0;
        static public int tickGap = 5;
        static public float targetPollutionAmt = 0.01f;
        static public int tickFindNewPollutionTarget = 2500;
        static public float val_waveGenUnitPer = 0.05f;
        static public int pollutionBaseGenCount => dataUtility.GetData(Find.World).pollutionGenCount;
        // dataUtility.dic_yyPollution 에서 value가 null이아니고 value.setuped가 true인 것들의 count
        static public int pollutionStartedTargetCount => dataUtility.dic_yyPollution.Count(x => x.Value != null && x.Value.setuped);
        
        KeyValuePair<int, yyCleanupData> kvpCleanup;
        

        public override void Tick(int currentTick)
        {
            tickGame = Find.TickManager.TicksGame;
            
            // make new pollution target object
            if (tickGame % tickFindNewPollutionTarget == 0)
            {
                worldData worldData = dataUtility.GetData(Find.World);
                // dataUtility.dic_yyPollution 의 밸류의 setuped 값이 true인 갯수가 val_targetCount 보다 크거나 같으면 스킵
                if (val_targetCount == -1 || worldData.pollutionGenCount < val_targetCount)
                {
                    WorldObject wo = null;
                    int seed = tickGame + Find.World.ConstantRandSeed;
                    
                    // find random faction base
                    if (Rand.RangeSeeded(0f, 100f, seed) <= 100f - val_nearPollutionPer)
                    {
                        // find new pollution target
                        List<WorldObject> ar = Find.World.worldObjects.AllWorldObjects.FindAll(a => a.def != WorldObjectDefOf.DestroyedSettlement && (!dataUtility.dic_yyPollution.ContainsKey(a) || !dataUtility.dic_yyPollution[a].setuped) && ar_faction.IndexOf(a.Faction.def) >= 0 && ar_faction_use[ar_faction.IndexOf(a.Faction.def)]);
                        if(ar.Count > 0) wo = ar[Rand.RangeSeeded(0, ar.Count, seed)];
                    }
                    
                    // gen near
                    if(wo == null)
                    {
                        // ar_faction_use 가 true인 것들중 랜덤한 인덱스를 구하고 ar_faction에 해당 인덱스의 값을 가져옴
                        List<WorldObject> ar = Find.World.worldObjects.AllWorldObjects.FindAll(a => ar_faction.IndexOf(a.Faction.def) >= 0 && ar_faction_use[ar_faction.IndexOf(a.Faction.def)]);
                        // fDef에 해당하는 팩션을 찾음
                        Faction faction = ar[Rand.RangeSeeded(0, ar.Count, seed)].Faction;
                        // 플레이어의 정착지들 중 랜덤한 하나를 선택
                        ar = Find.WorldObjects.AllWorldObjects.FindAll(x => x.Faction == Faction.OfPlayer);
                        int playerTile = ar[Rand.RangeSeeded(0, ar.Count, seed)].Tile;
                        int targetTile = TileFinder.RandomSettlementTileFor(faction, false,
                        (int x) => Find.WorldGrid[x].biome.canBuildBase && Find.WorldGrid.ApproxDistanceInTiles(x, playerTile) <= 25f);
                        if (targetTile != -1)
                        {
                            // faction에 해당하는 베이스 생성
                            wo = WorldObjectMaker.MakeWorldObject(WorldObjectDefOf.Settlement);
                            wo.SetFaction(faction);
                            wo.Tile = targetTile;
                            // 정착지 이름 설정
                            ((Settlement)wo).Name = SettlementNameGenerator.GenerateSettlementName((Settlement)wo);
                            Find.WorldObjects.Add(wo);
                        }
                        
                    }
                    
                    
                    if (wo != null)
                    {
                        if (val_useLog) Log.Message($"# gen data: {wo.Label}");
                        dataUtility.GetData(wo).setup();
                        worldData.pollutionGenCount += 1;
                    }
                }
                
            }
            
            // all pollution data tick
            if (tickGame % tickGap == 0)
            {
                // remove all destroyed worldObjects
                //dataUtility.dic_yyPollution.RemoveAll(a => a.Key == null || a.Value == null);

                // dataUtility.dic_yyPollution 를 순회하며 각데이터 for문
                foreach (KeyValuePair<WorldObject, yyPollutionData> kvp in dataUtility.dic_yyPollution)
                {
                    if (kvp.Value == null || !kvp.Value.setuped) continue;
                    //Log.Message($"pollution tick : {kvp.Key.Label}");
                    kvp.Value.tick();
                }
                
            }
            
            // dataUtility.dic_clearup for문
            for (int i = dataUtility.dic_clearup.Count - 1; i >= 0; i--)
            {
                kvpCleanup = dataUtility.dic_clearup.ElementAt(i);
                //if(core.val_useLog) Log.Message($"# for cleanup: {kvpCleanup.Key}, {kvpCleanup.Value}, {kvpCleanup.Value.setuped}");
                if (kvpCleanup.Value == null || !kvpCleanup.Value.setuped) continue;
                kvpCleanup.Value.tick();
            }

            



        }


        public static void PolluteWorldAtTile(int root, float pollutionAmount)
        {
            Tile tile = Find.WorldGrid[root];
            float pollution = tile.pollution;
            float targetPollution = tile.pollution + pollutionAmount;
            tile.pollution = Mathf.Clamp01(targetPollution);
            MapParent mapParent = Find.WorldObjects.MapParentAt(root);
            
            Map map = Current.Game.FindMap(root);
            if (pollution < 0.999f && map != null)
            {
                core.PolluteMapToPercent(map, tile.pollution);
            }
            Find.World.renderer.Notify_TilePollutionChanged(root);
        }



        public static float targetPollution = 0.5f;
        public static bool pollutionIgnoreRoot = false;
        public static bool pollutionAlram = true;
        private static List<IntVec3> tmpMapCells = new List<IntVec3>();
        public static void PolluteMapToPercent(Map m, float p)
        {
            //Log.Message($"core.PolluteMapToPercent: {p}");
            p = Mathf.Clamp01(p);

            // 감소해야하면 바닐라코드로 실행
            float mapPollutionPer = m.pollutionGrid.TotalPollutionPercent;
            // if (p < mapPollutionPer)
            // {
            //     PollutionUtility.PolluteMapToPercent(m, p, 1000);
            //     return;
            // }
            if (mapPollutionPer >= 1f)
            {
                m.TileInfo.pollution = p;
                return;
            }

            int tryCount = 100;
            if (p >= mapPollutionPer){
                // 증가
                while (m.pollutionGrid.TotalPollutionPercent < p * 0.983f && tryCount > 0)
                {
                    PollutionUtility.GrowPollutionAt(m.pollutionGrid.AllPollutableCells.RandomElement(), m, (int)Mathf.Clamp((Rand.RangeSeeded(100, 500, tickGame) * (p - mapPollutionPer) * 100f), 100, 1000));
                    tryCount--;
                }
            }else{
                // 감소
                tmpMapCells.Clear();
                tmpMapCells.AddRange(m.pollutionGrid.AllPollutableCells.InRandomOrder());
                int i = 0;
                while (m.pollutionGrid.TotalPollutionPercent > p && tryCount > 0 && i < tmpMapCells.Count)
                {
                    // 3회 반복 for문
                    for (int j = 0; j < 100 && i < tmpMapCells.Count; j++)
                    {
                        tmpMapCells[i].Unpollute(m);
                        i++;
                    }
                    
                }
                tmpMapCells.Clear();
            }
            
        }














    }










}