﻿using System;
using System.Collections.Generic;
using System.Linq;
using Verse;
using RimWorld;
using HarmonyLib;
using UnityEngine;
using System.Reflection;
using RimWorld.Planet;

namespace YayoPollution
{
    public class yyCleanupData : IExposable
    {
        int n;
        
        Tile _tile;
        Tile tile{
            get{
                if (_tile == null) {
                    _tile = Find.WorldGrid[tileN];
                }
                return _tile;
            }
        }
        
        
        // Data
        public float virtualPollution = 0f;
        public float wavePollution = 0f;
        public bool setuped = false;
        public int tileN;
        public List<int> radialMapping = new List<int>();
        public List<int> waveCurrentTick = new List<int>();
        
        

        // Data Save
        public void ExposeData()
        {
            // 데이터 저장, 로드
            Scribe_Values.Look<float>(ref virtualPollution, "virtualPollution", 0f);
            Scribe_Values.Look<float>(ref wavePollution, "wavePollution", 0f);
            Scribe_Values.Look(ref setuped, "setuped");
            Scribe_Values.Look(ref tileN, "tileN");
            if(core.val_useLog) Log.Message("yyCleanupData ExposeData tileN : " + tileN);
            Scribe_Collections.Look(ref radialMapping, "radialMapping", LookMode.Value);
            Scribe_Collections.Look(ref waveCurrentTick, "waveCurrentTick", LookMode.Value);

            if (Scribe.mode == LoadSaveMode.LoadingVars)
            {
                if (radialMapping == null) radialMapping = new List<int>();
                if (waveCurrentTick == null) waveCurrentTick = new List<int>();
            }
        }
        
        public void setParent(int _n)
        {
            n = _n;
        }


        List<tileAndDist> tmpTileAndDist = new List<tileAndDist>();
        public void setup(int _tileN, float limitDist){
            
            tileN = _tileN;
            virtualPollution = tile.pollution;
            
            // mapping 생성
            radialMapping = new List<int>();
            List<int> tmpTileSorted = new List<int>();
        
            for (int i = 0; i < Find.World.grid.TilesCount; i++)
            {
                if (Find.World.grid.tiles[i].WaterCovered) continue;
                tmpTileAndDist.Add(new tileAndDist(i, Find.World.grid.ApproxDistanceInTiles(tileN, i)));
            }
            if(limitDist > 0f) tmpTileAndDist.RemoveAll(a => a.dist > limitDist);
            tmpTileAndDist = tmpTileAndDist.OrderBy(a => a.dist).ToList();
            foreach (tileAndDist tad in tmpTileAndDist)
            {
                tmpTileSorted.Add(tad.tile);
            }
            // tmpTileSorted.Remove(tileN);
            radialMapping = tmpTileSorted;
            
            setuped = true;

            
            
            
        }
        
        public void remove()
        {
            if (core.val_useLog) Log.Message("CleanupData removed : " + tileN);
            setuped = false;
            dataUtility.RemoveCleanup(tileN);
            
        }
        
        public void tick(){
            
            // if(core.val_useLog) Log.Message("CleanupData tick : " + tileN);
            
            wavePollution += core.targetPollutionAmt * 4f;
            if(virtualPollution > 0f && wavePollution >= core.val_waveGenUnitPer * 4f){
                wavePollution -= core.val_waveGenUnitPer * 4f;
                waveCurrentTick.Add(0);
            }
            virtualPollution -= core.targetPollutionAmt * 2f;
            
            
            for(int i = waveCurrentTick.Count - 1; i >= 0; i--){
                if (waveCurrentTick[i] >= radialMapping.Count) {
                    // 제거
                    waveCurrentTick.RemoveAt(i);
                    continue;
                }
                core.PolluteWorldAtTile(radialMapping[waveCurrentTick[i]], -core.val_waveGenUnitPer * 4f);
                waveCurrentTick[i]++;
            }
            
            
            if(virtualPollution <= 0f && waveCurrentTick.Count == 0){
                remove();
                return;
            }
            
            
            
        }



    }
}
