﻿using System;
using System.Collections.Generic;
using System.Linq;
using Verse;
using RimWorld;
using HarmonyLib;
using UnityEngine;
using System.Reflection;
using RimWorld.Planet;

namespace YayoPollution
{
    public class yyPollutionData : IExposable
    {
        
        public WorldObject wo;
        Tile _tile;
        Tile tile{
            get{
                if (_tile == null) {
                    _tile = Find.WorldGrid[wo.Tile];
                }
                return _tile;
            }
        }
        
        
        // Data
        // public float virtualPollution = 0f;
        public float wavePollution = 0f;
        public bool setuped = false;
        public int tileN;
        // public int currentTick;
        public List<int> radialMapping = new List<int>();
        public List<int> waveCurrentTick = new List<int>();
        
        

        // Data Save
        public void ExposeData()
        {
            // 데이터 저장, 로드
            // Scribe_Values.Look<float>(ref virtualPollution, "virtualPollution", 0f);
            Scribe_Values.Look<float>(ref wavePollution, "wavePollution", 0f);
            Scribe_Values.Look(ref setuped, "setuped");
            Scribe_Values.Look(ref tileN, "tileN");
            // Scribe_Values.Look(ref currentTick, "currentTick");
            Scribe_Collections.Look(ref radialMapping, "radialMapping", LookMode.Value);
            Scribe_Collections.Look(ref waveCurrentTick, "waveCurrentTick", LookMode.Value);
            
            
            if (Scribe.mode == LoadSaveMode.LoadingVars)
            {
                if (radialMapping == null) radialMapping = new List<int>();
                if (waveCurrentTick == null) waveCurrentTick = new List<int>();
            }
        }

        public void setParent(WorldObject _wo)
        {
            wo = _wo;
        }


        List<tileAndDist> tmpTileAndDist = new List<tileAndDist>();
        public void setup(){
            
            tileN = wo.Tile;
            radialMapping = new List<int>();
            // mapping 생성
            List<int> tmpTileSorted = new List<int>();
        
            for (int i = 0; i < Find.World.grid.TilesCount; i++)
            {
                if (Find.World.grid.tiles[i].WaterCovered) continue;
                tmpTileAndDist.Add(new tileAndDist(i, Find.World.grid.ApproxDistanceInTiles(tileN, i)));
            }
            tmpTileAndDist = tmpTileAndDist.OrderBy(a => a.dist).ToList();
            foreach (tileAndDist tad in tmpTileAndDist)
            {
                tmpTileSorted.Add(tad.tile);
            }
            // tmpTileSorted.Remove(tileN);
            radialMapping = tmpTileSorted;
            
            setuped = true;

            Find.LetterStack.ReceiveLetter(
                string.Format("pollutionStart_t".Translate(), core.pollutionBaseGenCount + 1),
                string.Format("pollutionStart_d".Translate(), wo.Faction.Name.Colorize(wo.Faction.Color), wo.Label.Colorize(wo.Faction.Color), core.val_targetCount - core.pollutionBaseGenCount - 1),
                LetterDefOf.ThreatBig,
                new LookTargets(wo)
            );

            try
            {
                string str = ((Settlement)wo).Name;
                ((Settlement)wo).Name = $"{str} ({"CreatingPollution_t".Translate()})".Colorize(Color.red);
            }
            catch (Exception ex)
            {
                Log.Error(ex.Message);
            }
        }
        
        public void remove()
        {
            setuped = false;
            
            // 남은 오염생성 베이스 수
            int pollutionGenBasecount = 0;
            foreach (KeyValuePair<WorldObject, yyPollutionData> kvp in dataUtility.dic_yyPollution)
            {
                if (kvp.Value == null || !kvp.Value.setuped) continue;
                pollutionGenBasecount++;
            }
            
            
            if(pollutionGenBasecount == 0 && dataUtility.dic_world[Find.World].pollutionGenCount == core.val_targetCount){
                
                // all clear
                dataUtility.GetCleanupData(tileN).setup(tileN, -1f);
                
                Find.LetterStack.ReceiveLetter(
                    "clearAllPollution_t".Translate(),
                    "clearAllPollution_d".Translate(),
                    LetterDefOf.PositiveEvent,
                    new LookTargets(tileN)
                );
                // bgm을 credit 엔딩 bgm으로 변경
                Find.MusicManagerPlay.ForceSilenceFor(1f);
				Find.MusicManagerPlay.ForceStartSong(SongDefOf.EndCreditsSong, true);
                
                
            }else{
                
                // not all cleared
                dataUtility.GetCleanupData(tileN).setup(tileN, 30f);
                
                Find.LetterStack.ReceiveLetter(
                    "cleanupStart_t".Translate(),
                    string.Format("cleanupStart_d".Translate(), core.val_targetCount - core.pollutionBaseGenCount + core.pollutionStartedTargetCount),
                    LetterDefOf.PositiveEvent,
                    new LookTargets(tileN)
                );

                
                
            }
        }
        
        public void tick(){
            if (wo == null || wo.Destroyed || wo.def == WorldObjectDefOf.DestroyedSettlement || wo.Faction == Find.FactionManager.OfPlayer)
            {
                remove();
                return;
            }
            // virtualPollution += core.targetPollutionAmt;
            // if(virtualPollution > 1f) virtualPollution = 1f;
            
            // if (Mathf.Abs(virtualPollution - tile.pollution) >= 0.05f){
            //     core.PolluteWorldAtTile(tileN, virtualPollution - tile.pollution);
            // }
            // currentTick++;
            
            wavePollution += core.targetPollutionAmt;
            if(wavePollution >= core.val_waveGenUnitPer){
                wavePollution -= core.val_waveGenUnitPer;
                waveCurrentTick.Add(0);
            }
            
            for(int i = waveCurrentTick.Count - 1; i >= 0; i--){
                if (waveCurrentTick[i] >= radialMapping.Count) {
                    // 제거
                    waveCurrentTick.RemoveAt(i);
                    continue;
                }
                core.PolluteWorldAtTile(radialMapping[waveCurrentTick[i]], core.val_waveGenUnitPer);
                waveCurrentTick[i]++;
            }
            
            // for(int i = 0; i < core.val_waveSplit; i++){
            //     if (virtualPollution < (1f / core.val_waveSplit) * (i + 1)) break;
            //     while (waveCurrentTick.Count <= i)
            //     {
            //         waveCurrentTick.Add(0);
            //     }
            //     if (waveCurrentTick[i] >= radialMapping.Count) continue;
            //     core.PolluteWorldAtTile(radialMapping[waveCurrentTick[i]], 1f / core.val_waveSplit);
            //     waveCurrentTick[i]++;
            // }
            
            
            
        }



    }
}
